//ICC-AVR application builder : 2009-08-04  11:03:25
// Target : M128
// Crystal: 4.000Mhz
// Fuse: E-FF H-D9 L-E3
// 
// Programmed by Junghoon Kim
//
#include <iom128v.h>
#include <macros.h>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// IR. The IR is an 8-bit register that fetches instruction bytes from       B2 B1 B0 = XXX  (Don't care) 
// the received data stream and executes them to perform such                B5 B4 B3 = 000: Select none
// functions as setting up the operating mode for the chip (load the                  = 001: Select MDR0
// MDR) and data transfer among the various registers.                                = 010: Select MDR1
//                                                                                    = 011: Select DTR
//                                                                                    = 100: Select CNTR
//   +----+----+----+----+----+----+----+----+                                        = 101: Select OTR
//   | B7 | B6 | B5 | B4 | B3 | B2 | B1 | B0 |                                        = 110: Select STR
//   +----+----+----+----+----+----+----+----+                                        = 111: Select none
//                                                                              B7 B6 = 00: CLR  register 
//                                                                                    = 01: RD   register
//                                                                                    = 10: WR   register
//                                                                                    = 11: LOAD register
#define SELECT_MDR0 0b00001000     // Select MDR0
#define SELECT_MDR1 0b00010000     // Select MDR1
#define SELECT_DTR  0b00011000     // Select DTR
#define SELECT_CNTR 0b00100000     // Select CNTR
#define SELECT_OTR  0b00101000     // Select OTR
#define SELECT_STR  0b00110000     // Select STR

#define CLR_REG     0b00000000     // CLR  register
#define RD_REG      0b01000000     // RD   register
#define WR_REG      0b10000000     // WR   register
#define LOAD_REG    0b11000000     // LOAD register
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// MDR0. The MDR0 (Mode Register 0) is an 8-bit rean/write register that sets up the operating mode for LS7366R. The MDR0 is
// written into by executing the "write-to-MDR0" instruction via the instruction register. Upon power up MDR0 is cleared to zero. The
// following is a breakdown of the MDR bits:
//
//
//   +----+----+----+----+----+----+----+----+                                    
//   | B7 | B6 | B5 | B4 | B3 | B2 | B1 | B0 |                                    
//   +----+----+----+----+----+----+----+----+                                    
//
//  B1 B0 = 00: non-quadrature count mode. (A = clock, B = direction).
//        = 01: x1 quadrarure count mode (one count per quadrature cycle).
//        = 10: x2 quadrature count mode (two counts per quadrature cycle).
//        = 11: x4 quadrature count mode (four counts per quadrature cycle).
//  B3 B2 = 00: free-running count mode.
//        = 01: single-cycle count mode (counter disabled with carry borrow, re-enabled with reset or load).
//        = 10: range-limit count mode (up and down count-ranges are limited between DTR and zero,
//              respectively; counting freezes at these limits but resumes when direction reverse).
//        = 11: modulo-n count mode (input count clock frequency is divided by a factor of (n+1).
//  B5 B4 = 00: disable index
//        = 01: configure index as the "load CNTR"input (transfer DTR to CNTR).
//        = 10: configure index as the "reset CNTR" input (clears CNTR to 0).
//        = 11: configure index as the "load OTR"input(transfer CNTR to OTR).
//
//     B6 = 0: Asynchronous index
//        = 1: Synchoronous Index (oerridden in non-quadrature mode)
//     B7 = 0: Filter clock division factor = 1
//        = 1: Filter clock division factor = 2
//
#define NON_QUAD            0b00000000      // non-quadrature counter mode.
#define X1_QUAD             0b00000001      // x1 quadrature counter mode.
#define X2_QUAD             0b00000010      // x2 quadrature counter mode.
#define X4_QUAD             0b00000011      // x4 quadrature counter mode.
#define FREE_RUN            0b00000000      // free-running count mode.
#define SINGLE_CYCLE        0b00000100      // single-cycle count mode.
#define RANGE_LIMIT         0b00001000      // range-limit count mode.
#define MODULO_N            0b00001100      // modulo-n count mode.
#define DISABLE_INDEX       0b00000000      // disable index.
#define INDEX_AS_LOAD_CNTR  0b00010000      // configure index as the "load CNTR" input(clears CNTR to 0).
#define INDEX_AS_RESET_CNTR 0b00100000      // configure index as the "reset CNTR" input(clears CNTR to 0).
#define INDEX_AS_LOAD_OTR   0b00110000      // configure index as the "load OTR" input(transfer CNTR to OTR).
#define ASYCHRONOUS_INDEX   0b00000000      // Asynchronous index
#define SYNCHRONOUS_INDEX   0b01000000      // Synchoronous index
#define FILTER_CDF_1        0b00000000      // Filter clock division factor = 1
#define FILTER_CDF_2        0b10000000      // Filter clock division factor = 2
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// MDR1. The MDR1 (Mode Register 1) is an 8-bit read/write register which is appended to MDR0 for additional modes.
// Upon power-up MDR1 is cleared to zero
//
//   +----+----+----+----+----+----+----+----+                                    
//   | B7 | B6 | B5 | B4 | B3 | B2 | B1 | B0 |                                    
//   +----+----+----+----+----+----+----+----+                                    
//
//  B1 B0 = 00: 4-byte counter mode
//        = 01: 3-byte counter mode
//        = 10: 2-byte counter mode
//     B2 = 0: Enable counting
//        = 1: Disable conting
//     B3 =  : not used
//     B4 = 0: NOP
//        = 1: FLAG on IDX (B4 of STR)  -----+
//     B5 = 0: NOP                           |
//        = 1: FLAG on CMP (B5 of STR)       |  NOTE: Applicable to both
//     B6 = 0: NOP                           |  LFLAG/and DFLAG
//        = 1: FLAG on BW (B6 of STR)        |
//     B7 = 0: NOP                           |
//        = 1: FLAG on CY (B7 of STR)   -----+
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define FOUR_BYTE_COUNT_MODE    0b00000000       // 4-byte counter mode
#define THREE_BYTE_COUNT_MODE   0b00000001       // 3-byte counter mode
#define TWO_BYTE_COUNT_MODE     0b00000010       // 2-byte counter mode
#define ENABLE_COUNTING         0b00000000       // Enable counting
#define DISABLE_COUNTING        0b00000100       // Disable counting
#define FLAG_ON_IDX             0b00010000       // FLAG on IDX (B4 of STR)
#define FLAG_ON_CMP             0b00100000       // FLAG on CMP (B5 of STR)
#define FLAG_ON_BW              0b01000000       // FLAG on BW (B6 of STR)
#define FLAG_ON_CY              0b10000000       // FLAG on CY (B7 of STR)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define LS7366_SS_H_L  PORTB &= 0b11111110;    // Device Select or Start
#define LS7366_SS_L_H  PORTB |= 0b00000001;    // Device Unselect or End

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void port_init(void)
{
 PORTA = 0xFF;
 DDRA  = 0xFF;
 PORTB = 0x07;
 DDRB  = 0x07;
 PORTC = 0xFF; //m103 output only
 DDRC  = 0xFF;
 PORTD = 0xFF;
 DDRD  = 0xFF;
 PORTE = 0x00;
 DDRE  = 0x00;
 PORTF = 0xFF;
 DDRF  = 0xFF;
 PORTG = 0x00;
 DDRG  = 0x00;
}

//SPI initialize
// 
void spi_init(void)
{
 SPCR = 0x53; //setup SPI
 SPSR = 0x00; //setup SPI
}

//call this routine to initialize all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 XDIV  = 0x00; //xtal divider
 XMCRA = 0x00; //external memory
 port_init();
 spi_init();

 MCUCR = 0x00;
 EICRA = 0x00; //extended ext ints
 EICRB = 0x00; //extended ext ints
 EIMSK = 0x00;
 TIMSK = 0x00; //timer interrupt sources
 ETIMSK = 0x00; //extended timer interrupt sources
 SEI(); //re-enable interrupts
 //all peripherals are now initialized
}

unsigned char shift_data_io(unsigned char c) {
  
  SPDR = c;
  while(!(SPSR & (1<<SPIF))) ;
  return(SPDR);
}
//

void init_LS7366(void) {

 LS7366_SS_L_H                                 // Initialize SS
 
 LS7366_SS_H_L                                 // Set MDR0 X4 Count Mode
 shift_data_io(SELECT_MDR0 | WR_REG);
 shift_data_io(X4_QUAD | FREE_RUN | DISABLE_INDEX | SYNCHRONOUS_INDEX | FILTER_CDF_1 );
 LS7366_SS_L_H
 
 LS7366_SS_H_L                                 // Set MDR1 4 Bytes Counter Mode
 shift_data_io(SELECT_MDR1 | WR_REG);
 shift_data_io(FOUR_BYTE_COUNT_MODE | ENABLE_COUNTING);
 LS7366_SS_L_H
 
 LS7366_SS_H_L                                 // Clear(0) CNTR Register
 shift_data_io(SELECT_CNTR | CLR_REG);
 LS7366_SS_L_H
 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void main(void) {

 init_devices();
 init_LS7366();
 
 while (1) {
  LS7366_SS_H_L                           // Transfer CNTR to OTR in "parallel"
  shift_data_io(SELECT_OTR | LOAD_REG);  
  LS7366_SS_L_H

  LS7366_SS_H_L                           // Read 4 Bytes OTR Register
  shift_data_io(SELECT_OTR | RD_REG);     // 
  PORTF = shift_data_io(0x00);            // B31 - B24  MSB
  PORTA = shift_data_io(0x00);            // B23 - B16
  PORTC = shift_data_io(0x00);            // B15 - B08
  PORTD = shift_data_io(0x00);            // B07 - B00  LSB
  LS7366_SS_L_H
 
 }
}

/////////////////////////////// End of FILE //////////////////////////////////////////////////////////////////


